@@ -11,8 +11,34 @@ import sys
 import time
 
 from pymavlink import mavutil
+from MAVProxy.modules.lib.multiproc_util import MPDataLogChildTask
 
-def mavfft_display(mlog,timestamp_in_range):
+class MavFFT(MPDataLogChildTask):
+    '''A class used to launch `mavfft_display` in a child process'''
+
+    def __init__(self, *args, **kwargs):
+        '''
+        Parameters
+        ----------
+        mlog : DFReader
+            A dataflash or telemetry log
+        xlimits: MAVExplorer.XLimits
+            An object capturing timestamp limits
+        '''
+
+        super(MavFFT, self).__init__(*args, **kwargs)
+
+        # all attributes are implicitly passed to the child process 
+        self.xlimits = kwargs['xlimits']
+
+    # @override
+    def child_task(self):
+        '''Launch `mavfft_display`'''
+
+        # run the fft tool
+        mavfft_display(self.mlog, self.xlimits.timestamp_in_range)
+
+def mavfft_display(mlog, timestamp_in_range):
     '''display fft for raw ACC data in logfile'''
 
     '''object to store data about a single FFT plot'''
