@@ -96,6 +96,7 @@ class MiscModule(mp_module.MPModule):
         self.add_command('panic_autopilot', self.cmd_panic_autopilot, "panic autopilot")
         self.add_command('longloop_autopilot', self.cmd_longloop_autopilot, "cause long loop in autopilot")
         self.add_command('internalerror_autopilot', self.cmd_internalerror_autopilot, "cause internal error in autopilot")
+        self.add_command('dfu_boot', self.cmd_dfu_boot, "boot into DFU mode")
         self.add_command('batreset', self.cmd_battery_reset, "reset battery remaining")
         self.add_command('setorigin', self.cmd_setorigin, "set global origin")
         self.add_command('magsetfield', self.cmd_magset_field, "set expected mag field by field")
@@ -218,6 +219,10 @@ class MiscModule(mp_module.MPModule):
         '''Ask the autopilot to create an internal error'''
         self.cmd_dosomethingreallynastyto_autopilot(args, 'internal-error', 98)
 
+    def cmd_dfu_boot(self, args):
+        '''boot into DFU bootloader without hold'''
+        self.cmd_dosomethingreallynastyto_autopilot(args, 'DFU-boot-without-hold', 99)
+    
     def cmd_battery_reset(self, args):
         '''reset battery remaining'''
         mask = -1
